@@ -20,10 +20,14 @@ import enum
 import io
 import json
 import logging
-import re
 import zipfile
 
-import pkg_resources
+try:
+    from importlib import resources
+    if not hasattr(resources, 'files'):
+        import importlib_resources as resources
+except ImportError:
+    import importlib_resources as resources
 
 from sushy import exceptions
 from sushy.resources import constants
@@ -424,9 +428,13 @@ class JsonDataReader(AbstractDataReader):
     def get_data(self):
         """Gets JSON file from URI directly"""
         data = self._conn.get(path=self._path)
-
-        json_data = data.json() if data.content else {}
-
+        try:
+            json_data = data.json() if data.content else {}
+        except Exception as exc:
+            LOG.error("Unable to parse JSON in response. %(exc)s. The server "
+                      "returned:\n%(data)s",
+                      {'exc': exc, 'data': data.content})
+            raise
         return FieldData(data.status_code, data.headers, json_data)
 
 
@@ -483,9 +491,9 @@ class JsonPackagedFileReader(AbstractDataReader):
     def get_data(self):
         """Gets JSON file from packaged file denoted by path"""
 
-        with pkg_resources.resource_stream(self._resource_package_name,
-                                           self._path) as resource:
-            json_data = json.loads(resource.read().decode(encoding='utf-8'))
+        ref = resources.files(self._resource_package_name).joinpath(self._path)
+        with ref.open(encoding='utf-8') as fp:
+            json_data = json.load(fp)
             return FieldData(None, None, json_data)
 
 
@@ -578,6 +586,48 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
 
         return val
 
+    def _get_registry(self, identity, language='en', description='registry'):
+        """Get a registry with the given identity.
+
+        :param identity: The registry identity.
+        :param language: RFC 5646 language code for Message Registries.
+            Indicates language of registry to be used. Defaults to 'en'.
+        :param description: Human-readable description to use in logging.
+        :returns: the corresponding registry object or None.
+        """
+        registries = self._registries
+        if not registries:
+            LOG.info('No %s is available', description)
+            return None
+
+        for key, registry in registries.items():
+            if (registry
+                    and identity in (key, registry.identity)):
+                # NOTE(iurygregory): some registries may have "en-US"
+                # as their language, in this case we can check if the
+                # registry language starts with the requested language.
+                registry_language = registry.language.lower().split('-', 1)[0]
+                if (language != registry.language
+                        and language.lower() != registry_language):
+                    LOG.debug('Found %(descr)s but its language %(reg_lang)s '
+                              'does not match the requested %(lang)s',
+                              {'descr': description,
+                               'lang': language,
+                               'reg_lang': registry.language})
+                    continue
+
+                return registry
+
+        avail = ', '.join(f'{reg.identity} ({reg.language})'
+                          for reg in registries.values())
+        LOG.info('%(descr)s %(registry)s not available for language %(lang)s; '
+                 'available are: %(avail)s',
+                 {'descr': description,
+                  'registry': self._attribute_registry,
+                  'lang': language,
+                  'avail': avail})
+        return None
+
     def _parse_attributes(self, json_doc):
         """Parse the attributes of a resource.
 
@@ -601,21 +651,7 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
 
         :returns ETag or None
         """
-        etag = self._get_headers().get('ETag')
-        if not etag:
-            return None
-
-        # Note (arne_wiebalck): in case there is a weak Etag,
-        # return it with and without the qualifier to handle
-        # vendor implementations which may require one or the
-        # other (but should actually not use weak tags in the
-        # first place).
-        pattern = re.compile(r'^(W\/)("\w*")$')
-        match = pattern.match(etag)
-        if match:
-            return etag + ',' + match.group(2)
-
-        return etag
+        return self._get_headers().get('ETag')
 
     def _get_headers(self):
         """Returns the HTTP headers of the request for the resource.
