@@ -11,7 +11,7 @@
 #    under the License.
 
 # This is referred from Redfish standard schema.
-# http://redfish.dmtf.org/schemas/EthernetInterface.v1_3_0.json
+# https://redfish.dmtf.org/schemas/EthernetInterface.v1_4_0.json
 
 import logging
 
@@ -64,12 +64,12 @@ class EthernetInterfaceCollection(base.ResourceCollectionBase):
         are returned.
 
         :returns: dictionary in the format
-            {'aa:bb:cc:dd:ee:ff': sushy.STATE_ENABLED,
-            'aa:bb:aa:aa:aa:aa': sushy.STATE_DISABLED}
+            {'aa:bb:cc:dd:ee:ff': sushy.State.ENABLED,
+            'aa:bb:aa:aa:aa:aa': sushy.State.DISABLED}
         """
         mac_dict = {}
         for eth in self.get_members():
-            if eth.mac_address is not None and eth.status is not None:
-                if eth.status.health == res_cons.HEALTH_OK:
+            if eth.mac_address and eth.status is not None:
+                if eth.status.health == res_cons.Health.OK:
                     mac_dict[eth.mac_address] = eth.status.state
         return mac_dict
