@@ -11,9 +11,9 @@
 #    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 #    License for the specific language governing permissions and limitations
 #    under the License.
-
 from http import client as http_client
 import json
+import types
 from unittest import mock
 
 import sushy
@@ -24,6 +24,18 @@ from sushy.resources.manager import virtual_media
 from sushy.tests.unit import base
 
 
+class _FakeHTTPError(Exception):
+    """Mimics the object passed to is_credentials_required()."""
+
+    def __init__(self, status=400, json_body=None, detail=''):
+        super().__init__(detail)
+        self.detail = detail
+        self.response = types.SimpleNamespace(
+            status_code=status,
+            json=lambda: json_body if json_body is not None else {}
+        )
+
+
 class VirtualMediaTestCase(base.TestCase):
 
     def setUp(self):
@@ -173,9 +185,60 @@ class VirtualMediaTestCase(base.TestCase):
             ("/redfish/v1/Managers/BMC/VirtualMedia/Floppy1"),
             data={"Image": "https://www.dmtf.org/freeImages/Sardine.img",
                   "Inserted": True, "WriteProtected": False},
-            headers={"If-Match": 'W/"3d7b8a7360bf2941d","3d7b8a7360bf2941d"'})
+            headers={"If-Match": 'W/"3d7b8a7360bf2941d"'})
         self.assertTrue(self.sys_virtual_media._is_stale)
 
+    def test_is_transfer_protocol_required(self):
+        with open('sushy/tests/unit/json_samples/'
+                  'transfer_proto_required_error.json') as f:
+            response_obj = json.load(f)
+        response = mock.Mock(spec=['json', 'status_code'])
+        response.json.return_value = response_obj
+        error = exceptions.HTTPError('GET', 'VirtualMedia', response)
+        retval = self.sys_virtual_media.is_transfer_protocol_required(error)
+        self.assertTrue(retval)
+
+    def test_is_transfer_protocol_required_alt(self):
+        with open('sushy/tests/unit/json_samples/'
+                  'transfer_proto_required_error.json') as f:
+            response_obj = json.load(f)
+        response_obj['error']['code'] = 'Base.1.5.PropertyMissing'
+        response = mock.Mock(spec=['json', 'status_code'])
+        response.json.return_value = response_obj
+        error = exceptions.HTTPError('GET', 'VirtualMedia', response)
+        retval = self.sys_virtual_media.is_transfer_protocol_required(error)
+        self.assertTrue(retval)
+
+    def test_is_transfer_protocol_required_alt2(self):
+        with open('sushy/tests/unit/json_samples/'
+                  'transfer_proto_required_error2.json') as f:
+            response_obj = json.load(f)
+        response = mock.Mock(spec=['json', 'status_code'])
+        response.json.return_value = response_obj
+        error = exceptions.HTTPError('GET', 'VirtualMedia', response)
+        retval = self.sys_virtual_media.is_transfer_protocol_required(error)
+        self.assertTrue(retval)
+
+    def test_is_transfer_protocol_required_alt3(self):
+        with open('sushy/tests/unit/json_samples/'
+                  'transfer_proto_required_error3.json') as f:
+            response_obj = json.load(f)
+        response = mock.Mock(spec=['json', 'status_code'])
+        response.json.return_value = response_obj
+        error = exceptions.HTTPError('GET', 'VirtualMedia', response)
+        retval = self.sys_virtual_media.is_transfer_protocol_required(error)
+        self.assertTrue(retval)
+
+    def test_is_transfer_method_required(self):
+        with open('sushy/tests/unit/json_samples/'
+                  'transfer_method_required_error.json') as f:
+            response_obj = json.load(f)
+        response = mock.Mock(spec=['json', 'status_code'])
+        response.json.return_value = response_obj
+        error = exceptions.HTTPError('POST', 'VirtualMedia', response)
+        retval = self.sys_virtual_media.is_transfer_method_required(error)
+        self.assertTrue(retval)
+
     def test_eject_media_none(self):
         self.sys_virtual_media._actions.eject_media = None
         self.assertRaisesRegex(
@@ -223,7 +286,7 @@ class VirtualMediaTestCase(base.TestCase):
         self.sys_virtual_media._conn.patch.assert_called_once_with(
             ("/redfish/v1/Managers/BMC/VirtualMedia/Floppy1"),
             data={"Image": None, "Inserted": False},
-            headers={"If-Match": 'W/"3d7b8a7360bf2941d","3d7b8a7360bf2941d"'})
+            headers={"If-Match": 'W/"3d7b8a7360bf2941d"'})
         self.assertTrue(self.sys_virtual_media._is_stale)
 
     def test_eject_media_pass_empty_dict_415(self):
@@ -253,13 +316,16 @@ class VirtualMediaTestCase(base.TestCase):
         self.assertTrue(self.sys_virtual_media._is_stale)
 
     def test_set_verify_certificate(self):
+        self.conn.get.return_value.headers = {'Allow': 'GET,HEAD',
+                                              'ETag': '3d7b8a7360bf2941d'}
         with mock.patch.object(
                 self.sys_virtual_media, 'invalidate',
                 autospec=True) as invalidate_mock:
             self.sys_virtual_media.set_verify_certificate(True)
             self.sys_virtual_media._conn.patch.assert_called_once_with(
                 "/redfish/v1/Managers/BMC/VirtualMedia/Floppy1",
-                data={'VerifyCertificate': True})
+                data={'VerifyCertificate': True},
+                etag='3d7b8a7360bf2941d')
 
             invalidate_mock.assert_called_once_with()
 
@@ -277,3 +343,80 @@ class VirtualMediaTestCase(base.TestCase):
             redfish_version='1.0.2',
             registries=self.sys_virtual_media.registries,
             root=self.sys_virtual_media.root)
+
+    def test_detects_extended_info_username(self):
+        body = {
+            "error": {
+                "@Message.ExtendedInfo": [{
+                    "MessageId": "Base.1.12.ActionParameterMissing",
+                    "Message": "The action requires the parameter UserName to "
+                               "be present."
+                }]
+            }
+        }
+        err = _FakeHTTPError(json_body=body)
+        self.assertTrue(self.sys_virtual_media.is_credentials_required(err))
+
+    def test_detects_extended_info_password(self):
+        body = {
+            "error": {
+                "@Message.ExtendedInfo": [{
+                    "MessageId": "Base.1.15.ActionParameterMissing",
+                    "Message": "The action requires the parameter Password to "
+                               "be present."
+                }]
+            }
+        }
+        err = _FakeHTTPError(json_body=body)
+        self.assertTrue(self.sys_virtual_media.is_credentials_required(err))
+
+    def test_detects_plain_message_when_code_absent(self):
+        body = {"error": {
+            "message": "InsertMedia requires the parameter UserName"}}
+        err = _FakeHTTPError(json_body=body, detail="")
+        self.assertTrue(self.sys_virtual_media.is_credentials_required(err))
+
+    def test_detects_detail_fallback(self):
+        err = _FakeHTTPError(json_body={},
+                             detail="requires the parameter Password")
+        self.assertTrue(self.sys_virtual_media.is_credentials_required(err))
+
+    def test_preserves_legacy_code_based_detection(self):
+        body = {
+            "error": {
+                "code": "Base.1.10.GeneralError",
+                "message": "Action parameter missing: UserName"
+            }
+        }
+        err = _FakeHTTPError(json_body=body)
+        self.assertTrue(self.sys_virtual_media.is_credentials_required(err))
+
+    def test_no_trigger_on_unrelated_parameter(self):
+        body = {
+            "error": {
+                "@Message.ExtendedInfo": [{
+                    "MessageId": "Base.1.12.ActionParameterMissing",
+                    "Message": "The action requires the parameter Timeout to "
+                               "be present."
+                }]
+            }
+        }
+        err = _FakeHTTPError(json_body=body)
+        self.assertFalse(self.sys_virtual_media.is_credentials_required(err))
+
+    def test_no_trigger_on_unrelated_error(self):
+        body = {
+            "error": {
+                "@Message.ExtendedInfo": [{
+                    "MessageId": "Base.1.12.ResourceMissingAtURI",
+                    "Message": "Resource missing"
+                }]
+            }
+        }
+        err = _FakeHTTPError(json_body=body)
+        self.assertFalse(self.sys_virtual_media.is_credentials_required(err))
+
+    def test_no_trigger_when_no_clues(self):
+        err = _FakeHTTPError(json_body={"error": {"message": "Bad Request"}},
+                             detail="")
+        self.assertFalse(self.sys_virtual_media.is_credentials_required(err))
